SystemOrganization addCategory: #Everett!
SystemOrganization addCategory: #'Everett-Tests'!
SystemOrganization addCategory: #'Everett-TicTacToe'!

TestCase subclass: #EntityManagerTestCase
	instanceVariableNames: 'em'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

!EntityManagerTestCase methodsFor: 'running' stamp: 'EricShahkarami 7/2/2013 11:47'!
setUp 

	em := EntityManager new.! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/6/2013 18:50'!
testEntityMatches

	| e1 e2 pc1 bc1 bc2 c1 c2 c3 s1 s2 |
	
	e1 := em createEntity.
	e2 := em createEntity.

	pc1 := PositionComponent position: 3@7.
	bc1 := VelocityComponent velocity: 1@1.
	bc2 := VelocityComponent velocity: 2@0.
	
	em addComponent: pc1 toEntity: e1.
	em addComponent: bc1 toEntity: e1.
	em addComponent: bc2 toEntity: e2.
	
	c1 := Set newFrom: #(#position).
	c2 := Set newFrom: #(#velocity).
	c3 := Set newFrom: #(#position #velocity).

	s1 := Set new.
	s2 := Set new.
	s1 add: e1.
	s2 add: e1.
	s2 add: e2.
	
	self assert: ( (em entitiesContainingComponents: c1) = s1 ).
	self assert: ( (em entitiesContainingComponents: c2) = s2 ).
	self assert: ( (em entitiesContainingComponents: c3) = s1 )! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/6/2013 23:57'!
testRemoveComponent

	| e pc vc c1 c2 s |
	
	e := em createEntity.

	pc := PositionComponent position: 3@7.
	vc := VelocityComponent velocity: 1@1.
	
	em addComponent: pc toEntity: e.
	em addComponent: vc toEntity: e.
	em removeComponent: (PositionComponent identity) fromEntity: e.
		
	c1 := Set newFrom: #(#velocity).
	c2:= Set newFrom: #(#position velocity).
	
	s := Set new.
	s add: e.
	
	self assert: ( (em entitiesContainingComponents: c1) = s ).
	self assert: ( (em entitiesContainingComponents: c2) isEmpty = true ).! !

TestCase subclass: #GroupManagerTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

TestCase subclass: #SystemManagerTestCase
	instanceVariableNames: 'em sm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:51'!
setUp

	em := EntityManager new.
	sm := SystemManager new! !

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/7/2013 15:21'!
testSystemUpdate

	"Tests that e1 and e2 have their positions updated but not e3, which doesn't have a velocity component"

	| e1 e2 e3 pc1 pc2 pc3 vc1 vc2 |
	
	e1 := em createEntity.
	e2 := em createEntity.
	e3 := em createEntity.

	pc1 := PositionComponent position: 3@4.
	pc2 := PositionComponent position: 1@1.
	pc3 := PositionComponent position: 9@9.
	vc1 := VelocityComponent velocity: 1@1.
	vc2 := VelocityComponent velocity: 2@0.

	em addComponent: pc1 toEntity: e1.
	em addComponent: pc2 toEntity: e2.
	em addComponent: pc3 toEntity: e3.
	em addComponent: vc1 toEntity: e1.
	em addComponent: vc2 toEntity: e2.
	
	sm addSystem: (MovementSystem new).
	sm updateEntitiesOf: em afterDeltaTime: 1.
	
	self assert: ( (pc1 position) = (4@5) ).
	self assert: ( (pc2 position) = (3@1) ).
	self assert: ( (pc3 position) = (9@9) )! !

Object subclass: #EntityFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityFactory commentStamp: '<historical>' prior: 0!
I am a factory that creates entities with some specified component instances. Subclass me to specify which components.!

!EntityFactory class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/8/2013 14:34'!
createEntity: anEntityManager 

	| entity components |
	
	entity := anEntityManager createEntity.
	components := self obtainComponents.
	components do: [ :component | anEntityManager addComponent: component toEntity: entity ].
	^entity! !

!EntityFactory class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/8/2013 14:40'!
obtainComponents
	
	^self subclassResponsibility ! !

Object subclass: #EntityManager
	instanceVariableNames: 'entityTable nextAvailableEntity changedEntities inactiveEntities removedEntities groupManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityManager commentStamp: '<historical>' prior: 0!
I am responsible for creating, removing and tracking all entities and their components.
!

!EntityManager methodsFor: 'classify' stamp: 'eshkrm 7/7/2013 13:57'!
activateEntity: anEntity 

	inactiveEntities remove: anEntity.
	self changeEntity: anEntity! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:25'!
addComponent: aComponentInstance toEntity: anEntity

	(anEntity < nextAvailableEntity and: anEntity >= 0) ifTrue: 
	[  
		self entityAt: anEntity putComponent: aComponentInstance.
		self changeEntity: anEntity
	]! !

!EntityManager methodsFor: 'classify' stamp: 'eshkrm 7/6/2013 23:54'!
changeEntity: anEntity

	((changedEntities occurrencesOf: anEntity) > 1) ifFalse: [ changedEntities add: anEntity ]! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 19:34'!
changedEntitiesDo: aBlock

	(changedEntities asSet) do: [ :entity | aBlock value: entity ]! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:39'!
clearChangedEntities

	self changedEntitiesDo: [ :entity | changedEntities remove: entity ]! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:39'!
clearRemovedEntities

	self removedEntitiesDo: [ :entity | removedEntities remove: entity ]! !

!EntityManager methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 09:38'!
component: aComponentIdentifier fromEntity: anEntity

	| componentsOfEntity |
	
	componentsOfEntity := self entityAt: anEntity.
	^(componentsOfEntity at: aComponentIdentifier)! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:27'!
createEntity

	entityTable at: nextAvailableEntity put: (Dictionary new).	
	self changeEntity: nextAvailableEntity.
	nextAvailableEntity := nextAvailableEntity + 1.
	^(nextAvailableEntity - 1)
! !

!EntityManager methodsFor: 'classify' stamp: 'eshkrm 7/7/2013 13:49'!
deactivateEntity: anEntity

	inactiveEntities add: anEntity! !

!EntityManager methodsFor: 'testing' stamp: 'EricShahkarami 7/5/2013 08:34'!
doesEntity: anEntity containComponents: aSetOfComponents

	| entityComponents |
	
	entityComponents := (self entityAt: anEntity) keys asSet.
	^( (aSetOfComponents intersection: entityComponents) = aSetOfComponents )! !

!EntityManager methodsFor: 'accessing' stamp: 'eshkrm 7/6/2013 18:50'!
entitiesContainingComponents: aSetOfComponents

	^( self entitiesSelect: [ :entity | self doesEntity: entity containComponents: aSetOfComponents ] )! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 23:51'!
entitiesDo: aBlock

	entityTable keysDo: [ :entity | aBlock value: entity ].! !

!EntityManager methodsFor: 'enumerate' stamp: 'EricShahkarami 7/2/2013 11:22'!
entitiesSelect: aBlock

	| selectEntities |
	
	selectEntities := Set new.
	entityTable keysDo: [ :entity | (aBlock value: entity) ifTrue: [ selectEntities add: entity ]  ].
	^selectEntities! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity
	
	^(entityTable at: anEntity)! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity putComponent: aComponentInstance 
		
	(entityTable at: anEntity) at: (aComponentInstance class identity) put: aComponentInstance. ! !

!EntityManager methodsFor: 'accessing' stamp: 'ShruthiSudhanva 7/8/2013 17:07'!
groupManager

	^groupManager.! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 13:53'!
inactiveEntitiesDo: aBlock 

	inactiveEntities do: [ :entity | aBlock value: entity ]! !

!EntityManager methodsFor: 'initialize-release' stamp: 'ShruthiSudhanva 7/8/2013 14:46'!
initialize 
	
	super initialize.
	entityTable := Dictionary new.
	changedEntities := Bag new.
	inactiveEntities := Set new.
	removedEntities := Set new.
	nextAvailableEntity := 0.! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:25'!
removeComponent: aComponentIdentifier fromEntity: anEntity

	| components |
	
	components := self entityAt: anEntity.
	components removeKey: aComponentIdentifier.
	self changeEntity: anEntity
! !

!EntityManager methodsFor: 'add-remove' stamp: 'EricShahkarami 7/1/2013 11:17'!
removeEntity: anEntity	
	"Removes entity from entityTable and places entity in removed pile."	
	
	entityTable removeKey: anEntity.
	removedEntities add: anEntity
	! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 19:15'!
removedEntitiesDo: aBlock 

	removedEntities do: [ :entity | aBlock value: entity ]! !

Object subclass: #EverettComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettComponent commentStamp: '<historical>' prior: 0!
I hold data for entities. Systems modify me.!

EverettComponent subclass: #CellComponent
	instanceVariableNames: 'button boardIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!CellComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/9/2013 17:54'!
identity

	^#cell! !

!CellComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/9/2013 15:20'!
position: aPosition 

	^(self new position: aPosition)! !

!CellComponent methodsFor: 'accessing' stamp: 'eshkrm 7/9/2013 17:46'!
boardIndex
	
	^boardIndex! !

!CellComponent methodsFor: 'accessing' stamp: 'eshkrm 7/9/2013 17:47'!
boardIndex: anIndex

	boardIndex := anIndex! !

!CellComponent methodsFor: 'accessing' stamp: 'eshkrm 7/9/2013 17:41'!
button

	^button! !

!CellComponent methodsFor: 'accessing' stamp: 'eshkrm 7/9/2013 17:42'!
color: aColor

	button color: aColor! !

!CellComponent methodsFor: 'initialize-release' stamp: 'eshkrm 7/9/2013 17:42'!
initialize

	button := TicTacToeButton new.
! !

!CellComponent methodsFor: 'geometry' stamp: 'eshkrm 7/9/2013 17:42'!
position: aPosition

	button position: aPosition! !

!CellComponent methodsFor: 'testing' stamp: 'eshkrm 7/9/2013 21:07'!
wasClicked

	^(button wasClicked)! !

!EverettComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 6/27/2013 13:52'!
identity
	^self subclassResponsibility! !

EverettComponent subclass: #PositionComponent
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!PositionComponent commentStamp: '<historical>' prior: 0!
I am a component that holds the position of an entity.!

!PositionComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:09'!
identity
	^#position! !

!PositionComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
position: aPosition

	^(self new position: aPosition)! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
position

	^position! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:02'!
position: aPosition 
	
	position := aPosition! !

EverettComponent subclass: #VelocityComponent
	instanceVariableNames: 'velocity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!VelocityComponent commentStamp: '<historical>' prior: 0!
I am a component that holds the velocity of an entity.!

!VelocityComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:27'!
identity 	
	^#velocity! !

!VelocityComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
velocity: aVelocity

	^(self new velocity: aVelocity)! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity

	^velocity ! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity: aVelocity
	
	velocity := aVelocity! !

Object subclass: #EverettSystem
	instanceVariableNames: 'updateInterval lastProcessed interestedComponents interestedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettSystem commentStamp: '<historical>' prior: 0!
I am system that processes entities.
!

!EverettSystem class methodsFor: 'instance creation' stamp: 'EricShahkarami 6/27/2013 14:20'!
identity
	^self subclassResponsibility ! !

!EverettSystem methodsFor: 'initialize-release' stamp: 'eshkrm 7/6/2013 19:12'!
initialize

	updateInterval := 0.
	lastProcessed := 0.
	interestedComponents := Set new.
	interestedEntities := Set new! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 15:17'!
interestedEntities

	^interestedEntities! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 16:14'!
lastProcessed 
	^lastProcessed! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 16:18'!
setLastProcessed: alastProcessed
	lastProcessed := alastProcessed! !

!EverettSystem methodsFor: 'processing' stamp: 'AnkuAdhikari 7/11/2013 16:08'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt
"default method"
					
	
		interestedEntities do: [ :entity | self updateEntity: entity from: anEntityManager afterDeltaTime: dt ].
	! !

!EverettSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 11:12'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	^self subclassResponsibility! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 16:28'!
updateInterval
	^updateInterval! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 15:59'!
updateInterval: anUpdateInterval
	
	updateInterval := anUpdateInterval ! !

!EverettSystem methodsFor: 'testing' stamp: 'AnkuAdhikari 7/9/2013 16:55'!
verifyEntity: anEntity from: anEntityManager

	(anEntityManager doesEntity: anEntity containComponents: interestedComponents) 
	ifTrue: 
	[  
		(interestedEntities includes: anEntity) ifFalse: [ interestedEntities add: anEntity ].
	]
	ifFalse:
	[ 
		(interestedEntities includes: anEntity) ifTrue: [ interestedEntities remove: anEntity ]
	]! !

EverettSystem subclass: #InputHandlerSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!InputHandlerSystem methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/9/2013 20:55'!
initialize
	
	super initialize.
	interestedComponents add: (CellComponent identity)! !

!InputHandlerSystem methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/9/2013 21:01'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	| cc |
	
	cc := anEntityManager component: (CellComponent identity) fromEntity: anEntity.
		
! !

EverettSystem subclass: #MovementSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!MovementSystem commentStamp: '<historical>' prior: 0!
I am a system that updates the position of entities.!

!MovementSystem class methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:55'!
identity 

	^#movement! !

!MovementSystem methodsFor: 'initialize-release' stamp: 'EricShahkarami 7/5/2013 11:19'!
initialize
	
	super initialize.
	interestedComponents add: (PositionComponent identity); add: (VelocityComponent identity).! !

!MovementSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 15:56'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| pc vc |
	
	pc := anEntityManager component: (PositionComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	pc position: ( pc position + (dt * (vc velocity)) )! !

Object subclass: #GroupManager
	instanceVariableNames: 'groups entityManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/8/2013 23:00'!
addComponent: aComponentInstance toGroup: aGroupIdentifier

	self entitiesInGroup: aGroupIdentifier do: [ :entity | entityManager addComponent: aComponentInstance toEntity: entity ]! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/8/2013 23:00'!
addEntity: anEntity toGroup: aGroupIdentifier

	(groups at: aGroupIdentifier) add: anEntity! !

!GroupManager methodsFor: 'modify-groups' stamp: 'eshkrm 7/8/2013 22:55'!
createGroup: aGroupIdentifier

	groups at: aGroupIdentifier put: Set new! !

!GroupManager methodsFor: 'modify-groups' stamp: 'eshkrm 7/8/2013 22:55'!
createGroup: aGroupIdentifier withEntities: aCollectionOfEntities

	groups at: aGroupIdentifier put: aCollectionOfEntities asSet! !

!GroupManager methodsFor: 'accessing' stamp: 'eshkrm 7/8/2013 23:00'!
entitesInGroup: aGroupIdentifier

	^(groups at: aGroupIdentifier)! !

!GroupManager methodsFor: 'enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier collect: aBlock

	^( (groups at: aGroupIdentifier) collect: [ :entity | aBlock value: entity ] )! !

!GroupManager methodsFor: 'enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier detect: aBlock

	^( (groups at: aGroupIdentifier) detect: [ :entity | aBlock value: entity ] ).	! !

!GroupManager methodsFor: 'enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier do: aBlock

	(groups at: aGroupIdentifier) do: [ :entity | aBlock value: entity ]! !

!GroupManager methodsFor: 'enumerate' stamp: 'eshkrm 7/8/2013 23:02'!
entitiesInGroup: aGroupIdentifier select: aBlock

	^( (groups at: aGroupIdentifier) select: [ :entity | aBlock value: entity ] )! !

!GroupManager methodsFor: 'initialize-release' stamp: 'eshkrm 7/8/2013 22:55'!
initialize 	

	super initialize.
	groups := Dictionary new! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/8/2013 22:59'!
removeComponent: aComponentIdentifier fromGroup: aGroupIdentifier

	self entitiesInGroup: aGroupIdentifier do: [ :entity | entityManager removeComponent: aComponentIdentifier fromEntity: entity ]
! !

!GroupManager methodsFor: 'add-remove' stamp: 'ShruthiSudhanva 7/7/2013 20:15'!
removeEntity: anEntity fromGroup: aGroupIdentfier

	(groups at: aGroupIdentfier) remove: anEntity. 	! !

!GroupManager methodsFor: 'modify-groups' stamp: 'eshkrm 7/8/2013 22:55'!
removeGroup: aGroupIdentifier

	groups removeKey: aGroupIdentifier
! !

Object subclass: #SystemManager
	instanceVariableNames: 'priorities updateInterval interestedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!SystemManager commentStamp: '<historical>' prior: 0!
I manage all running and inactive systems.!

!SystemManager methodsFor: 'add-remove' stamp: 'EricShahkarami 7/1/2013 15:18'!
addSystem: aSystemInstance 
	
	self addSystem: aSystemInstance withPriority: 0 ! !

!SystemManager methodsFor: 'add-remove' stamp: 'eshkrm 7/7/2013 21:01'!
addSystem: aSystemInstance withPriority: aPriority
	
	self priorityAt: aPriority putSystem: aSystemInstance.


	! !

!SystemManager methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/11/2013 16:28'!
haveSystem: system updateEntitiesFrom: anEntityManager afterDeltaTime: dt

	"calls updateEntitiesOf:afterDeltaTime: . Also it would call all the resolve methods for that system and check updateInterval of that system to make sure it can process entities."
	|lp new_lp ui|
	self resolveChangedEntitiesFrom: anEntityManager withSystem: system.
	self resolveInactiveEntitiesFrom: anEntityManager withSystem: system.	
	self resolveRemovedEntitiesFrom: anEntityManager withSystem: system.
		
	lp:= system lastProcessed. 
	new_lp:= lp + dt.
	ui:= system updateInterval.
	(new_lp >= ui) ifTrue: 
	[ 
		system setLastProcessed: 0.
		system updateEntitiesOf: anEntityManager afterDeltaTime: dt
	]
	ifFalse:
	[  
		system setLastProcessed: new_lp.
	]! !

!SystemManager methodsFor: 'initialize-release' stamp: 'eshkrm 7/8/2013 22:39'!
initialize
	
	super initialize.
	priorities := SortedCollection sortBlock: [ :p1 :p2 | p1 key >= p2 key ]! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/7/2013 20:16'!
priorityAt: aPriority

	| systemsWithAPriority |
	
	systemsWithAPriority := priorities detect: [ :assoc | assoc key = aPriority ].
	^(systemsWithAPriority value)! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/7/2013 21:00'!
priorityAt: aPriority putSystem: aSystemInstance

	| systemsWithAPriority |
	
	(priorities includes: aPriority) ifFalse: [ priorities add: ( aPriority->(Dictionary new) ) ].
	
	systemsWithAPriority := self priorityAt: aPriority.
	systemsWithAPriority at: (aSystemInstance class identity) put: aSystemInstance! !

!SystemManager methodsFor: 'add-remove' stamp: 'ShruthiSudhanva 7/7/2013 22:02'!
removeSystem: aSystemIdentifier

	| systems |

	systems := self systemsWithPriorityDetect: [ :systemsWithPriority | systemsWithPriority includesKey: aSystemIdentifier ].
	systems ifNotNil: [ systems removeKey: aSystemIdentifier ]
	! !

!SystemManager methodsFor: 'private' stamp: 'AnkuAdhikari 7/11/2013 16:00'!
resolveChangedEntitiesFrom: anEntityManager withSystem: aSystem
	

	anEntityManager changedEntitiesDo: [ :entity | aSystem verifyEntity: entity from: anEntityManager ]! !

!SystemManager methodsFor: 'private' stamp: 'AnkuAdhikari 7/11/2013 16:00'!
resolveInactiveEntitiesFrom: anEntityManager withSystem: aSystem
	
	anEntityManager inactiveEntitiesDo: [ :entity | (aSystem interestedEntities includes: entity) ifTrue: [aSystem interestedEntities remove: entity ] ]! !

!SystemManager methodsFor: 'private' stamp: 'AnkuAdhikari 7/11/2013 16:01'!
resolveRemovedEntitiesFrom: anEntityManager withSystem: aSystem 
	
	anEntityManager removedEntitiesDo: [ :entity | (aSystem  interestedEntities includes: entity) ifTrue: [ aSystem  interestedEntities remove: entity ] ]! !

!SystemManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 21:23'!
systemsDo: aBlock

	"Run block on systems in order of their priority"
	priorities do: [ :assoc | (assoc value) do: [ :system | aBlock value: system ] ]! !

!SystemManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 21:14'!
systemsWithPriorityDetect: aBlock

	| systemsWithPriority |

	systemsWithPriority := priorities detect: [ :assoc | aBlock value: (assoc value) ].
	^systemsWithPriority ! !

!SystemManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 21:23'!
systemsWithPriorityDo: aBlock 
	
	"Run block on systems with same priority."
	priorities do: [ :assoc | aBlock value: (assoc value ) ]! !

!SystemManager methodsFor: 'processing' stamp: 'AnkuAdhikari 7/9/2013 16:29'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	self systemsDo: [ :system |  self haveSystem: system updateEntitiesFrom: anEntityManager afterDeltaTime: dt ].
	
	anEntityManager clearChangedEntities.
	anEntityManager clearRemovedEntities.
	! !

Object subclass: #TicTacToeGame
	instanceVariableNames: 'em window cells'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/9/2013 20:48'!
createBoard

	1 to: 3 do: [ :i | 1 to: 3 do: [ :j | self createCellAt: i at: j ] ]! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/9/2013 20:51'!
createCellAt: i at: j

	"Create a cell for position (i,j) and add it to my on-screen representation at the appropriate screen position."
	| b cc e origin |
	
	cc := CellComponent new.
	cc boardIndex: i@j.
	
	b := cc button.
	origin := window innerBounds origin.
	window addMorph: b.
	b position: ( (i -1) * b width) @ ((j - 1) * b height ) + origin.
	
 	e := em createEntity.
	em addComponent: cc toEntity: e.! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/9/2013 20:53'!
initialize

	em := EntityManager new.

	window := BorderedMorph new.
	window bounds: (5 @ 5 extent: (64 * 3) @ (64 * 3) + (2 * window borderWidth)).
	self createBoard! !

!TicTacToeGame methodsFor: 'gameplay' stamp: 'eshkrm 7/9/2013 17:56'!
play

	window openInWorld! !

SimpleButtonMorph subclass: #TicTacToeButton
	instanceVariableNames: 'wasClicked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!TicTacToeButton methodsFor: 'button' stamp: 'eshkrm 7/9/2013 16:26'!
doButtonAction 

	wasClicked := true.
	self color: (Color orange)! !

!TicTacToeButton methodsFor: 'initialize-release' stamp: 'eshkrm 7/9/2013 16:25'!
initialize 

	super initialize.
	self label: ''.
	self borderWidth: 2.
	bounds := 0@0 corner: 64@64.
	self useSquareCorners.
	wasClicked := false
	! !

!TicTacToeButton methodsFor: 'event handling' stamp: 'eshkrm 7/9/2013 16:26'!
mouseDown: anEvent

	oldColor := self fillStyle. 
	actWhen == #buttonDown
		ifTrue: [ self doButtonAction]! !

!TicTacToeButton methodsFor: 'accessing' stamp: 'eshkrm 7/9/2013 21:08'!
wasClicked

	^wasClicked ! !
